% Klee–Minty cube

% Article: Finite convergence into a convex polytope via facet reflections
% Journal: Applications of Mathematics
% Authors: Dinesh B. Ekanayake, Douglas LaFountain, Boris Petracovici.

clear;clc;

p=36; % The dimensions of the Klee–Minty cube
x=ones(1,p-1); b1=ones(p,1);
for j=1:p-1
    x(j)=2^(p+1-j);
    b1(j)=5^j;
end
b1(p)=5^p;
b2=zeros(p,1);
b=[b1;b2];
w1=eye(p);
for i=1:p-1
    w1(p+1-i,1:p-i)=x(1,i:p-1);
end
w2=-eye(p);
w=[w1;w2]; 
v=diag(b);
v=v(:,1:p);
b=-b;

x0=250*ones(p,1); 
tic;
[B,k]=ReflectIn(x0,w,b,v,0);
t1=toc;
disp('Reflection algorithm: ' )
disp(['Number of iterations = ', num2str(k)])
disp(['Elapsed time = ', num2str(t1)])
disp(['Solution x1 = '])
disp(B)

x0=[x0;1000]; w=[w -ones(2*p,1)]; 
f=zeros(p+1,1); f(end)=1;
Aeq=[]; beq=[]; lb=zeros(1,p+1); lb(1,p+1)=-inf;ub=[];

disp('Interior point algorithm: ' )
options = optimoptions(@linprog,'Algorithm','interior-point','Display','iter');

tic;
x2 = linprog(f,w,-b,Aeq,beq,lb,ub,options);
t2=toc;

disp(['Elapsed time = ', num2str(t2)])
disp(['Solution x2 = '])
disp(x2(1:end-1))
if x2(end)>0
    disp('The point is outside the polytope.')
end

options = optimoptions(@linprog,'Display','iter');

tic;
x3 = linprog(f,w,-b,Aeq,beq,lb,ub,options);
t3=toc;

disp('Dual simplex algorithm: ' )
disp(['Elapsed time = ', num2str(t3)])
disp(['solution x3 = '])
disp(x3(1:end-1))

