% Ackley function

% Article: Finite convergence into a convex polytope via facet reflections
% Journal: Applications of Mathematics
% Authors: Dinesh B. Ekanayake, Douglas LaFountain, Boris Petracovici.

clear;clc;

subplot(1,2,1) % Grid of points
u = 100:2:200; t = meshgrid(u); t=t(:)'; 
s = meshgrid(u)'; s=s(:)';
plot(t,s,'.k')
axis([80 220 80 220]);

subplot(1,2,2)
% Regular convex hexagon
v0=[0 6 9 6 0 -3];
v1=[-3*sqrt(3) -3*sqrt(3) 0 3*sqrt(3) 3*sqrt(3) 0];
plot(polyshape(v0,v1))
hold on;

% Reflections of the grid from the algorithm
w=[0 1;sqrt(3) 1;sqrt(3) -1;0 -1;-sqrt(3) -1; -sqrt(3) 1]; 
b=-3*[sqrt(3);3*sqrt(3);3*sqrt(3);sqrt(3);sqrt(3);sqrt(3)];
v=[0 3*sqrt(3);9 0;9 0;0 -3*sqrt(3);-3 0;-3 0];
x0=[t;s];
tic
[B,k]=ReflectIn(x0,w,b,v,0);
toc

% Reflected points
plot(B(1,:),B(2,:),'.','LineWidth',1,'Color',[0.2 0.2 0.2]);
hold off

% Approximate global optimal 
f=-20.0*exp(-0.2*sqrt(0.5*((B(1,:)-2).^2 + (B(2,:)-2).^2))) ...
   -exp(0.5*(cos(2*pi.*(B(1,:)-2))+cos(2*pi.*(B(2,:)-2))))+exp(1)+20;
[~,i0]=min(f);
x0=B(:,i0);

% Global optimal
fun=@(x) -20.0*exp(-0.2*sqrt(0.5*((x(1)-2).^2 + (x(2)-2).^2))) ...
   - exp(0.5*(cos(2*pi.*(x(1)-2)) + cos(2*pi.*(x(2)-2)))) + exp(1) + 20;

Aeq=[]; beq=[]; lb=[]; ub=[]; nonlcon=[];
options = optimoptions(@fmincon,'Display','iter-detailed');

x1 = fmincon(fun,x0,w,-b,Aeq,beq,lb,ub,nonlcon,options);

disp('Reflection algorithm: ')
x1

xi=[100 100 200 200; 100 200 200 100];

% Optimization of Ackley's function
fun=@(x) -20.0*exp(-0.2*sqrt(0.5*((x(1)-2).^2 + (x(2)-2).^2))) ...
   - exp(0.5*(cos(2*pi.*(x(1)-2)) + cos(2*pi.*(x(2)-2)))) + exp(1) + 20;

Aeq=[]; beq=[]; lb=[]; ub=[]; nonlcon=[];

x2=zeros(2,4); x3=zeros(2,4);

for i=1:4
    x0=xi(:,i);

    options = optimoptions(@fmincon,'Display','off');
    x2(:,i)  = fmincon(fun,x0,w,-b,Aeq,beq,lb,ub,nonlcon,options);

    options = optimoptions(@fmincon,'Algorithm','sqp','Display','off');
    x3(:,i) = fmincon(fun,x0,w,-b,Aeq,beq,lb,ub,nonlcon,options);
end

disp('interior-point method : ')
x2

disp('SQP method')
x3

