function [B,k]=ReflectIn(A,w,b,v,Display)
% Reflection Algorithm
% Enter points as a matrix A=[x0,x1,x2,...].
% Write the half spaces as w'x+b<=0.
% Enter a set of points on the hyperplanes. One can find such a set by 
% setting the coefficients of each hyperplane to zero for all but one.

% Article: Finite convergence into a convex polytope via facet reflections
% Journal: Applications of Mathematics
% Authors: Dinesh B. Ekanayake*,
%          Douglas LaFountain,
%          Boris Petracovici.

% *-Corresponding author

if nargin==3
    error('Enter a set of points on the hyperplanes');
elseif nargin==4
    Display=0; 
end

% find the norm of w.
wn=sqrt(sum(w.*w,2));

% normalize the hyperplanes.
w=w./wn;
b=b./wn;
B=[]; k=0; 

while ~isempty(A)
    % find the distance with sign.
    d=(w*A+b);

    % find the largest postive row.
    [dmax,j]=max(d);
    j=j(dmax>0);

    % B gives Points in the region.
    B=[B A(:,dmax<=0)];
    A0=A(:,dmax>0);

    if ~isempty(A0)
        k=k+1;
        n=(w(j,:))';
        v0=(v(j,:))';

        A=A0-2*n.*diag((A0-v0)'*n)';
        if Display==1
            plot([A0(1,:),A(1,:)],[A0(2,:), A(2,:)],'-o',...
                'LineWidth',1,'Color',[0.6 0.6 0.6]);
            hold on;
        end
    else
        A=A0;
    end
end
if Display==1
    hold off
end